//
//  FirstViewController.m
//  DMSDemo
//
//  Created by Ao dian on 15/6/24.
//  Copyright (c) 2015年 aodianyun. All rights reserved.
//

#import "FirstViewController.h"
#import "SecondViewController.h"
#import "DMSManager.h"

@interface FirstViewController ()
@property (weak, nonatomic) IBOutlet UITextField *txtAddress;
@property (weak, nonatomic) IBOutlet UITextField *txtPubKey;
@property (weak, nonatomic) IBOutlet UITextField *txtSubKey;
@property (weak, nonatomic) IBOutlet UITextField *txtClientId;
@property (weak, nonatomic) IBOutlet UIButton *btnConnect;
@property (weak, nonatomic) IBOutlet UIButton *btnDisconnect;
@property (weak, nonatomic) IBOutlet UITextField *txtSubTopic;
@property (weak, nonatomic) IBOutlet UIButton *btnSubscribe;
@property (weak, nonatomic) IBOutlet UIButton *btnUnsubscribe;

@end

@implementation FirstViewController
- (IBAction)TextField_DidEndOnExit:(UITextField *)sender {
    [sender resignFirstResponder];
}
- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    self.txtClientId.text = [UIDevice currentDevice].identifierForVendor.UUIDString;
    SecondViewController* ctrl = [self.parentViewController.childViewControllers objectAtIndex:1];
    [ctrl loadView];
    [self setViewState:NO];
}
- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}
- (void) addMessage:(NSString*) msg
{
    dispatch_async(dispatch_get_main_queue(), ^{
        SecondViewController* ctrl = [self.parentViewController.childViewControllers objectAtIndex:1];
        [ctrl addMessage:msg];
    });
}
- (void) showMessage:(NSString*) msg
{
    UIAlertView *alter = [[UIAlertView alloc] initWithTitle:nil message:msg delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alter show];
}
- (void) setViewState:(BOOL) isConnected
{
    if(isConnected) {
        self.txtAddress.enabled = false;
        self.txtPubKey.enabled = false;
        self.txtSubKey.enabled = false;
        self.txtClientId.enabled = false;
        self.btnConnect.enabled = false;
        self.btnDisconnect.enabled = true;
        self.btnSubscribe.enabled = true;
        self.btnUnsubscribe.enabled = true;
        self.txtSubTopic.enabled = true;
    } else {
        self.txtAddress.enabled = true;
        self.txtPubKey.enabled = true;
        self.txtSubKey.enabled = true;
        self.txtClientId.enabled = true;
        self.btnConnect.enabled = true;
        self.btnDisconnect.enabled = false;
        self.btnSubscribe.enabled = false;
        self.btnUnsubscribe.enabled = false;
        self.txtSubTopic.enabled = false;
    }
    [[self.parentViewController.childViewControllers objectAtIndex:1] setViewState:isConnected];
}
- (IBAction)btnConnectClick:(UIButton *)sender {
    if(self.txtClientId.text.length>0 &&
       self.txtAddress.text.length>0 &&
       self.txtPubKey.text.length>0 &&
       self.txtSubKey.text.length>0) {
        self.btnConnect.enabled = false;
        [DMSManager initWithClientId:self.txtClientId.text];
        [[DMSManager client] setMessageHandler:^(MQTTMessage *message) {
            [self addMessage:[[NSString alloc] initWithFormat:@"recv : %@",message.payloadString]];
        }];
        [[DMSManager client] setDisconnectionHandler:^(NSUInteger code) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self setViewState:NO];
            });
        }];
        int result = [[DMSManager client] connectToHost:self.txtAddress.text
                                 withPubKey:self.txtPubKey.text
                                     subKey:self.txtSubKey.text
                          completionHandler:^(MQTTConnectionReturnCode code) {
                              if (code == ConnectionAccepted) {
                                  dispatch_async(dispatch_get_main_queue(), ^{
                                      [self setViewState:YES];
                                  });
                              } else {
                                  dispatch_async(dispatch_get_main_queue(), ^{
                                      [[DMSManager client] disconnectWithCompletionHandler:nil];
                                      [self setViewState:NO];
                                      [self showMessage:@"Connection refused"];
                                  });
                              }
                          }];
        if(result != DMS_SUCCESS) {
            [self setViewState:NO];
            [self showMessage:@"Connection failed"];
        }
    }
    [self.view endEditing:YES];
}
- (IBAction)btnDisconnectClick:(UIButton *)sender {
    self.btnDisconnect.enabled = false;
    [[DMSManager client] disconnectWithCompletionHandler:^(NSUInteger code) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self setViewState:NO];
        });
    }];
    [self.view endEditing:YES];
}
- (IBAction)btnSubscribeClick:(UIButton *)sender {
    if(self.txtSubTopic.text.length > 0) {
        [[DMSManager client] subscribe:self.txtSubTopic.text
                  withCompletionHandler:^(NSArray *grantedQos) {
                      dispatch_async(dispatch_get_main_queue(), ^{
                          [self showMessage:[[NSString alloc] initWithFormat:@"%@%@", @"subscribed to topic ",self.txtSubTopic.text]];
                      });
                  }];
    }
    [self.view endEditing:YES];
}
- (IBAction)btnUnsubscribeClick:(UIButton *)sender {
    if(self.txtSubTopic.text.length > 0) {
        [[DMSManager client] unsubscribe:self.txtSubTopic.text
                    withCompletionHandler:^{
                        dispatch_async(dispatch_get_main_queue(), ^{
                            [self showMessage:[[NSString alloc] initWithFormat:@"%@%@", @"unsubscribed to topic ",self.txtSubTopic.text]];
                        });
                    }];
    }
    [self.view endEditing:YES];
}

@end
